<?php

namespace Drupal\taxonomy_custom_controller\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\taxonomy\TermInterface;
use Drupal\taxonomy_custom_controller\Event\TaxonomyCustomControllerEvents;
use Drupal\taxonomy_custom_controller\Event\TermPageBuildEvent;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Provides custom controller for taxonomy routes.
 */
class TaxonomyCustomController implements ContainerInjectionInterface {

  /**
   * The event dispatcher.
   *
   * @var \Symfony\Component\EventDispatcher\EventDispatcherInterface
   */
  protected $eventDispatcher;

  /**
   * Constructs a new TaxonomyTerm object.
   *
   * @param \Symfony\Component\EventDispatcher\EventDispatcherInterface $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(EventDispatcherInterface $event_dispatcher) {
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('event_dispatcher')
    );
  }

  /**
   * The build process for term pages.
   *
   * @param \Drupal\taxonomy\TermInterface $taxonomy_term
   *   The taxonomy term entity.
   *
   * @return array|null
   *   The result render array.
   */
  public function build(TermInterface $taxonomy_term) {
    $event = new TermPageBuildEvent($taxonomy_term);
    $this->eventDispatcher->dispatch(TaxonomyCustomControllerEvents::PAGE_BUILD, $event);

    $build = $event->getBuildArray();
    if (!empty($build)) {
      return $build;
    }

    return views_embed_view('taxonomy_term', 'page_1', $taxonomy_term->id());
  }

}
